#########
# 解説1
#########
import ctypes
import tisgrabber as tis
import cv2
import numpy as np
import xml.etree.ElementTree as ET #XMLファイルを解析する

ic = ctypes.cdll.LoadLibrary("./tisgrabber_x64.dll")
tis.declareFunctions(ic)

#########
# 解説2
#########
"" "コールバック関数に渡されるユーザーデータの例。" ""
class CallbackUserdata(ctypes.Structure):
    def __init__(self):    # インスタンス生成時に自動的に呼ばれるメソッド
        self.Value1 = 0
        self.Value2 = 0
        self.camera = None      # grabberオブジェクトを参照

#########
# 解説3
#########
def FrameCallback(hGrabber, pBuffer, framenumber, pData):
    """ これはコールバック関数の例です
    画像はtest.jpgに保存され、pData.Value1が1つインクリメントされます。
     ：param：hGrabber：これはグラバーオブジェクトへの実際のポインター(使用禁止)
     ：param：pBuffer：最初のピクセルのバイトへのポインタ
     ：param：framenumber：ストリームが開始されてからのフレーム数
     ：param：pData：追加のユーザーデータ構造へのポインター
    """
    #CallbackUserdataが機能しているか確認
    
    pData.Value1 = pData.Value1 + 1
    Width = ctypes.c_long()
    Height = ctypes.c_long()
    BitsPerPixel = ctypes.c_int()
    colorformat = ctypes.c_int()
    ic.IC_GetImageDescription(hGrabber, Width, Height, BitsPerPixel,colorformat)


    # バッファサイズを計算しピクセルあたりのバイト数と
    # numpy配列のデータ型を取得します。
    elementsperpixel = 1
    dtype = np.uint8
    if colorformat.value == tis.SinkFormats.Y800:
        elementsperpixel = 1  # 1 byte per pixel
    if colorformat.value == tis.SinkFormats.Y16:
        dtype = np.uint16
        elementsperpixel = 1  # 1 uint16 per pixel
    if colorformat.value == tis.SinkFormats.RGB24:
        elementsperpixel = 3  # BGR format, 3 bytes
    if colorformat.value == tis.SinkFormats.RGB32:
        elementsperpixel = 4  # BGRA format, 4 bytes
    
    # バッファサイズを計算
    buffer_size = Width.value * Height.value * int(float(BitsPerPixel.value) / 8.0)

    # イメージデータを取得
    imagePtr = ic.IC_GetImagePtr(hGrabber)
    imagedata = ctypes.cast(imagePtr,
                            ctypes.POINTER(ctypes.c_ubyte *
                                            buffer_size))

    # NumPy配列で表された画像にする
    image = np.ndarray(buffer=imagedata.contents,
                        dtype=dtype,
                        shape=(Height.value,
                                Width.value,
                                elementsperpixel))



    # リアルタイムでOpenCV処理
    threshold = 80 # 二値化の閾値の設定
    image = cv2.flip(image, 0) #画像反転
    # OpenCVライブラリを使って収縮処理
    image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY) #モノクロ化
    ret, img_THRESH_BINARY = cv2.threshold(image, threshold, 255, cv2.THRESH_BINARY) #二値化
    resized_img = cv2.resize(img_THRESH_BINARY,(640, 480)) #表示を縮小
    cv2.imshow('Window', resized_img)
    cv2.waitKey(10)

#########
# 解説4
#########
Userdata = CallbackUserdata()
# 関数ポインタを作成
Callbackfuncptr = ic.FRAMEREADYCALLBACK(FrameCallback)

ic.IC_InitLibrary(0)

#ダイアログ画面を表示
hGrabber = tis.openDevice(ic)


#########
# 解説5
#########
#デバイスが有効か確認
if(ic.IC_IsDevValid(hGrabber)):
    ## XMLファイルを解析
    tree = ET.parse('device.xml') 
    # XMLを取得
    root = tree.getroot()
    
    # device.xmlからカラーフォーマットを取得
    colorFormat=""
    for name in root.iter('videoformat'):
        colorFormat=name.text
    
    # IC_SetFormatにカラーフォーマットをセット
    if ('Y800' in colorFormat):
        ic.IC_SetFormat(hGrabber, tis.SinkFormats.Y800)

    if ('Y16' in colorFormat):
        ic.IC_SetFormat(hGrabber, tis.SinkFormats.Y16)

    if ('RGB24' in colorFormat):
        ic.IC_SetFormat(hGrabber, tis.SinkFormats.RGB24)

    if ('RGB32' in colorFormat):
        ic.IC_SetFormat(hGrabber, tis.SinkFormats.RGB32)

    print(colorFormat)

    ic.IC_SetFrameReadyCallback(hGrabber, Callbackfuncptr, Userdata)
    #連続モードでは、フレームごとにコールバックが呼び出されます。
    ic.IC_SetContinuousMode(hGrabber, 0) 
    ic.IC_SetDefaultWindowPosition(hGrabber, 0)
    ic.IC_SetWindowPosition(hGrabber, 0,0, 640, 480)
    ic.IC_StartLive(hGrabber, 0) #ライブスタート開始　引数：0の時非表示、引数：1の時表示
    
    ic.IC_MsgBox(tis.T("OKで停止します。"), tis.T("Callback"))
    ic.IC_StopLive(hGrabber)
else:
    ic.IC_MsgBox(tis.T("No device opened"), tis.T("Callback"))

ic.IC_ReleaseGrabber(hGrabber)
